@@ -1,5 +1,3 @@
-# $Id: IO_Poll.pm 2173 2007-02-18 07:47:52Z rcaputo $
-
 # IO::Poll event loop bridge for POE::Kernel.  The theory is that this
 # will be faster for large scale applications.  This file is
 # contributed by Matt Sergeant (baud).
@@ -8,7 +6,7 @@
 package POE::Loop::IO_Poll;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2173 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.284'; # NOTE - Should be #.### (three decimal places)
 
 # Include common signal handling.
 use POE::Loop::PerlSignals;
@@ -16,6 +14,17 @@ use POE::Loop::PerlSignals;
 # Everything plugs into POE::Kernel;
 package POE::Kernel;
 
+=for poe_tests
+
+sub skip_tests {
+  return "IO::Poll is not 100% compatible with $^O" if $^O eq "MSWin32";
+  return "IO::Poll tests require the IO::Poll module" if (
+    do { eval "use IO::Poll"; $@ }
+  );
+}
+
+=cut
+
 use strict;
 
 # Be sure we're using a contemporary version of IO::Poll.  There were
@@ -79,18 +88,23 @@ sub loop_attach_uidestroy {
 }
 
 #------------------------------------------------------------------------------
-# Maintain time watchers.
+# Maintain time watchers.  For this loop, we simply save the next
+# event time in a scalar.  loop_do_timeslice() will use the saved
+# value.  A "paused" time watcher is just a timeout for some future
+# time.
+
+my $_next_event_time = time();
 
 sub loop_resume_time_watcher {
-  # does nothing
+  $_next_event_time = $_[1];
 }
 
 sub loop_reset_time_watcher {
-  # does nothing
+  $_next_event_time = $_[1];
 }
 
 sub loop_pause_time_watcher {
-  # does nothing
+  $_next_event_time = time() + 3600;
 }
 
 # A static function; not some object method.
@@ -214,14 +228,15 @@ sub loop_do_timeslice {
   # there are any.  If nothing is waiting, set the timeout for some
   # constant number of seconds.
 
-  my $now = time();
+  my $timeout = $_next_event_time;
 
-  my $timeout = $self->get_next_event_time();
+  my $now = time();
   if (defined $timeout) {
     $timeout -= $now;
     $timeout = 0 if $timeout < 0;
   }
   else {
+    die "shouldn't happen" if ASSERT_DATA;
     $timeout = 3600;
   }
 
@@ -235,8 +250,6 @@ sub loop_do_timeslice {
     );
   }
 
-  my @filenos = %poll_fd_masks;
-
   if (TRACE_FILES) {
     foreach (sort { $a<=>$b} keys %poll_fd_masks) {
       my @types;
@@ -259,99 +272,101 @@ sub loop_do_timeslice {
     }
   }
 
-  # Avoid looking at filehandles if we don't need to.  -><- The added
-  # code to make this sleep is non-optimal.  There is a way to do this
-  # in fewer tests.
+  # Avoid looking at filehandles if we don't need to.
+  # TODO The added code to make this sleep is non-optimal.  There is a
+  # way to do this in fewer tests.
 
-  if ($timeout or @filenos) {
+  if (scalar keys %poll_fd_masks) {
 
     # There are filehandles to poll, so do so.
 
-    if (@filenos) {
-      # Check filehandles, or wait for a period of time to elapse.
-      my $hits = IO::Poll::_poll($timeout * 1000, @filenos);
+    # Check filehandles, or wait for a period of time to elapse.
+    my $hits = IO::Poll::_poll($timeout * 1000, my @results = %poll_fd_masks);
 
-      if (ASSERT_FILES) {
-        if ($hits < 0) {
-          POE::Kernel::_trap("<fh> poll returned $hits (error): $!")
-            unless ( ($! == EINPROGRESS) or
-                     ($! == EWOULDBLOCK) or
-                     ($! == EINTR)
-                   );
-        }
-      }
-
-      if (TRACE_FILES) {
-        if ($hits > 0) {
-          POE::Kernel::_warn "<fh> poll hits = $hits\n";
-        }
-        elsif ($hits == 0) {
-          POE::Kernel::_warn "<fh> poll timed out...\n";
-        }
-      }
-
-      # If poll has seen filehandle activity, then gather up the
-      # active filehandles and synchronously dispatch events to the
-      # appropriate handlers.
-
-      if ($hits > 0) {
-
-        # This is where they're gathered.
-
-        my (@rd_ready, @wr_ready, @ex_ready);
-        while (@filenos) {
-          my ($fd, $got_mask) = splice(@filenos, 0, 2);
-          next unless $got_mask;
-
-          my $watch_mask = $poll_fd_masks{$fd};
-          if ( $watch_mask & POLLRDNORM and
-               $got_mask & (POLLRDNORM | POLLHUP | POLLERR | POLLNVAL)
-             ) {
-            if (TRACE_FILES) {
-              POE::Kernel::_warn "<fh> enqueuing read for fileno $fd";
-            }
-
-            push @rd_ready, $fd;
-          }
-
-          if ( $watch_mask & POLLWRNORM and
-               $got_mask & (POLLWRNORM | POLLHUP | POLLERR | POLLNVAL)
-             ) {
-            if (TRACE_FILES) {
-              POE::Kernel::_warn "<fh> enqueuing write for fileno $fd";
-            }
-
-            push @wr_ready, $fd;
-          }
-
-          if ( $watch_mask & POLLRDBAND and
-               $got_mask & (POLLRDBAND | POLLHUP | POLLERR | POLLNVAL)
-             ) {
-            if (TRACE_FILES) {
-              POE::Kernel::_warn "<fh> enqueuing expedite for fileno $fd";
-            }
-
-            push @ex_ready, $fd;
-          }
-        }
-
-        @rd_ready and $self->_data_handle_enqueue_ready(MODE_RD, @rd_ready);
-        @wr_ready and $self->_data_handle_enqueue_ready(MODE_WR, @wr_ready);
-        @ex_ready and $self->_data_handle_enqueue_ready(MODE_EX, @ex_ready);
+    if (ASSERT_FILES) {
+      if ($hits < 0) {
+        POE::Kernel::_trap("<fh> poll returned $hits (error): $!")
+          unless ( ($! == EINPROGRESS) or
+                   ($! == EWOULDBLOCK) or
+                   ($! == EINTR) or
+                   ($! == 0)      # SIGNAL_PIPE strangeness
+                 );
       }
     }
 
+    if (TRACE_FILES) {
+      if ($hits > 0) {
+        POE::Kernel::_warn "<fh> poll hits = $hits\n";
+      }
+      elsif ($hits == 0) {
+        POE::Kernel::_warn "<fh> poll timed out...\n";
+      }
+    }
+
+    # If poll has seen filehandle activity, then gather up the
+    # active filehandles and synchronously dispatch events to the
+    # appropriate handlers.
+
+    if ($hits > 0) {
+
+      # This is where they're gathered.
+
+      my (@rd_ready, @wr_ready, @ex_ready);
+      my %poll_fd_results = @results;
+      while (my ($fd, $got_mask) = each %poll_fd_results) {
+        next unless $got_mask;
+
+        my $watch_mask = $poll_fd_masks{$fd};
+        if (
+          $watch_mask & POLLRDNORM and
+          $got_mask & (POLLRDNORM | POLLHUP | POLLERR | POLLNVAL)
+        ) {
+          if (TRACE_FILES) {
+            POE::Kernel::_warn "<fh> enqueuing read for fileno $fd";
+          }
+
+          push @rd_ready, $fd;
+        }
+
+        if (
+          $watch_mask & POLLWRNORM and
+          $got_mask & (POLLWRNORM | POLLHUP | POLLERR | POLLNVAL)
+        ) {
+          if (TRACE_FILES) {
+            POE::Kernel::_warn "<fh> enqueuing write for fileno $fd";
+          }
+
+          push @wr_ready, $fd;
+        }
+
+        if (
+          $watch_mask & POLLRDBAND and
+          $got_mask & (POLLRDBAND | POLLHUP | POLLERR | POLLNVAL)
+        ) {
+          if (TRACE_FILES) {
+            POE::Kernel::_warn "<fh> enqueuing expedite for fileno $fd";
+          }
+
+          push @ex_ready, $fd;
+        }
+      }
+
+      @rd_ready and $self->_data_handle_enqueue_ready(MODE_RD, @rd_ready);
+      @wr_ready and $self->_data_handle_enqueue_ready(MODE_WR, @wr_ready);
+      @ex_ready and $self->_data_handle_enqueue_ready(MODE_EX, @ex_ready);
+    }
+  }
+  elsif ($timeout) {
+
     # No filehandles to poll on.  Try to sleep instead.  Use sleep()
     # itself on MSWin32.  Use a dummy four-argument select() everywhere
     # else.
 
+    if ($^O eq 'MSWin32') {
+      sleep($timeout);
+    }
     else {
-      if ($^O eq 'MSWin32') {
-        sleep($timeout);
-      }
-      else {
-        CORE::select(undef, undef, undef, $timeout);
-      }
+      CORE::select(undef, undef, undef, $timeout);
     }
   }
 
@@ -382,7 +397,7 @@ __END__
 
 =head1 NAME
 
-POE::Loop::IO_Poll - a bridge that supports IO::Poll from POE
+POE::Loop::IO_Poll - a bridge that allows POE to be driven by IO::Poll
 
 =head1 SYNOPSIS
 
@@ -390,13 +405,13 @@ See L<POE::Loop>.
 
 =head1 DESCRIPTION
 
-This class is an implementation of the abstract POE::Loop interface.
-It follows POE::Loop's public interface exactly.  Therefore, please
-see L<POE::Loop> for its documentation.
+POE::Loop::IO_Poll implements the interface documented in L<POE::Loop>.
+Therefore it has no documentation of its own.  Please see L<POE::Loop>
+for more details.
 
 =head1 SEE ALSO
 
-L<POE>, L<POE::Loop>, L<IO::Poll>
+L<POE>, L<POE::Loop>, L<IO::Poll>, L<POE::Loop::PerlSignals>
 
 =head1 AUTHORS & LICENSING
 
@@ -404,3 +419,6 @@ Please see L<POE> for more information about authors, contributors,
 and POE's licensing.
 
 =cut
+
+# rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
